from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.workspace.testing import FTW_WORKSPACE_FUNCTIONAL_TESTING
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import TEST_USER_ID
from unittest2 import TestCase


class TestMoveItems(TestCase):

    layer = FTW_WORKSPACE_FUNCTIONAL_TESTING

    def setUp(self):
        portal = self.layer['portal']
        setRoles(portal, TEST_USER_ID, ['Reviewer', 'Manager'])

    @browsing
    def test_move_items_moves_items(self, browser):

        # setup folder / items
        source = create(Builder('folder')
                         .titled('Source Folder'))

        target = create(Builder('folder')
                         .titled('Target Folder'))

        create(Builder('file')
               .titled('The File')
               .within(source)
               .attach_file_containing('content', name='file.txt'))

        # open source containing file
        browser.login(SITE_OWNER_NAME).open()
        browser.visit(source, view='folder_contents')

        # select file and hit move button
        browser.css('#cb_the-file').first.node.checked = True
        browser.find('Move Items').click()

        browser.fill({'Destination': target})
        browser.find('Move').click()

        self.assertTrue(
            target.get('the-file'),
            '"The File" should be located in the target folder.')



from DateTime import DateTime
from datetime import datetime
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.workspace.testing import FTW_WORKSPACE_FUNCTIONAL_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.testing.z2 import Browser
from pyquery import PyQuery as pq
from unittest2 import TestCase


class TestOverviewTab(TestCase):

    layer = FTW_WORKSPACE_FUNCTIONAL_TESTING

    def setUp(self):
        portal = self.layer['portal']
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)

        self.workspace = create(Builder('workspace')
            .titled('Workspace')
            .having(description='Description'))

        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD, ))

    def test_overview_tab_available(self):
        view = self.workspace.restrictedTraverse('tabbedview_view-overview')

        self.assertIsNotNone(view, 'Overview tab is no available.')

    @browsing
    def test_activity_stream_is_visible(self, browser):
        browser.login().open(self.workspace,
                             view='tabbedview_view-overview')
        self.assertTrue(browser.css('.activity .events').first)

    def test_overview_description(self):
        self.browser.open(
            '%s/tabbedview_view-overview' % self.workspace.absolute_url())
        doc = pq(self.browser.contents)

        self.assertIn(self.workspace.Description(),
            doc('.textbox')[0].text_content(),
            'Description not found')

    def test_overview_base_catalog_result(self):
        file1 = create(Builder('file')
            .within(self.workspace)
            .having(modificationDate=DateTime() - 1)
            .attach_file_containing('DATA', name='dummy.pdf'))

        file2 = create(Builder('file')
            .within(self.workspace)
            .having(modificationDate=DateTime() - 2)
            .attach_file_containing('DATA', name='dummy.pdf'))

        view = self.workspace.restrictedTraverse('tabbedview_view-overview')

        self.assertEquals([self.workspace.id, file1.id, file2.id],
            [brain.getId for brain in view.catalog()],
            'Wrong default order')

    def test_display_subfolders(self):
        folder = create(Builder('TabbedViewFolder')
            .within(self.workspace)
            .titled('SubFolder'))

        self.browser.open(
            '%s/tabbedview_view-overview' % self.workspace.absolute_url())
        doc = pq(self.browser.contents)
        self.assertEquals(folder.Title(),
            doc('.overview-left-column .box ul li a.rollover').text())

    def test_show_ten_recently_modified_items(self):
        for i in range(1, 15):
            create(Builder('file')
                .within(self.workspace))

        view = self.workspace.restrictedTraverse('tabbedview_view-overview')

        self.assertEquals(10, len(view.recently_modified()),
            'Display only the ten most recent changes')

    def test_save_description(self):
        create(Builder('file')
            .within(self.workspace)
            .having(description='<b>Bold description</b>')
            .attach_file_containing('DATA', name='dummy.pdf'))
        brain = self.workspace.getFolderContents()[0]

        view = self.workspace.restrictedTraverse('tabbedview_view-overview')

        self.assertEquals('Bold description', view.get_description(brain))

    def test_overview_show_search_description(self):
        view = self.workspace.restrictedTraverse('tabbedview_view-overview')

        self.assertFalse(view.show_search_results())

        view.request['searchable_text'] = 'something'
        self.assertTrue(view.show_search_results())

    def test_overview_type_class_file(self):
        view = self.workspace.restrictedTraverse('tabbedview_view-overview')
        create(Builder('file')
            .within(self.workspace)
            .attach_file_containing('DATA', name='dummy.pdf'))
        brain = self.workspace.getFolderContents()[0]

        self.assertEquals('', view.type_class(brain))

    def test_overview_type_class_other(self):
        view = self.workspace.restrictedTraverse('tabbedview_view-overview')
        create(Builder('document')
            .within(self.workspace))
        brain = self.workspace.getFolderContents()[0]

        self.assertEquals('contenttype-document',
            view.type_class(brain))

    def test_overview_generate_date_today(self):
        view = self.workspace.restrictedTraverse('tabbedview_view-overview')
        now = datetime(2013, 6, 3, 11, 0)

        datestring = '2013-06-03 09:00:00'
        self.assertEquals('label_today', view.generate_date(datestring, now))

    def test_overview_generate_date_yesterday(self):
        view = self.workspace.restrictedTraverse('tabbedview_view-overview')
        now = datetime(2013, 6, 3, 11, 0)

        datestring = '2013-06-02 09:00:00'
        self.assertEquals('label_yesterday',
            view.generate_date(datestring, now))

    def test_overview_generate_date_older(self):
        view = self.workspace.restrictedTraverse('tabbedview_view-overview')
        now = datetime(2013, 6, 3, 11, 0)

        datestring = '2013-03-01 11:00:00'
        self.assertEquals('01.03.2013', view.generate_date(datestring, now))



from ftw.upgrade import UpgradeStep
from ftw.workspace.interfaces import IWorkspace
import logging

LOG = logging.getLogger('ftw.workspace.upgrades')


class IndexWorkspaceCreator(UpgradeStep):

    def __call__(self):
        query = {'object_provides': IWorkspace.__identifier__}
        self.catalog_reindex_objects(query, idxs=['SearchableText'])



from DateTime import DateTime
from ftw.tabbedview.browser import listing
from ftw.table import helper as table_helper
from ftw.workspace import _
from ftw.workspace.browser import helper
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
import datetime


class ListingHelper(object):

    def get_description(self, file_):
        description = file_.getObject().Description()
        # make sure there is no html in description
        transforms = getToolByName(self.context, 'portal_transforms')
        result = transforms.convertTo('text/plain', description)
        if result is None:
            return ''
        else:
            return result.getData()

    def type_class(self, brain):
        """ Returns the contenttype or mimetype class for sprites.
        """
        plone_utils = getToolByName(self.context, 'plone_utils')
        if not brain.getIcon:
            return 'contenttype-' + plone_utils.normalizeString(
                brain.portal_type)
        return ''

    def get_icon(self, document):
        return helper.icon(document, "")


class OverviewTab(listing.CatalogListingView, ListingHelper):
    """Overview tab for workspace"""

    overview_template = ViewPageTemplateFile("overview.pt")
    sort_on = 'modified'
    sort_reverse = True
    show_menu = False
    show_selects = False

    columns = ({'column': 'Title',
                'sort_index': 'sortable_title',
                'column_title': _(u'label_eventstab_title'),
                'transform': table_helper.linked},

               {'column': 'modified',
                'column_title': _(u'column_modified',
                                  default=u'modified'),
                'transform': table_helper.readable_date},

               {'column': 'Creator',
                'sort_index': 'sortable_creator',
                'column_title': _(u'label_eventstab_creator'),
                'transform': helper.readable_author}, )

    def __init__(self, context, request):
        super(OverviewTab, self).__init__(context, request)
        catalog = getToolByName(self.context, 'portal_catalog')

        if 'searchable_text' in self.request:
            searchable_text = self.request.get('searchable_text')
            if len(searchable_text):
                kwargs = {}

                if not searchable_text.endswith('*'):
                    searchable_text += '*'

                kwargs['SearchableText'] = searchable_text
                kwargs['sort_on'] = 'modified'
                kwargs['sort_order'] = 'reverse'
                results = catalog(
                    path='/'.join(self.context.getPhysicalPath()), **kwargs)
                self.contents = helper.group_by_date(results)

    def template(self):
        if self.filter_text != '':
            return super(OverviewTab, self).template()
        else:
            return self.overview_template()

    def catalog(self, types=None, depth=-1, sort_on='modified',
                sort_order='reverse'):

        query = dict(
            path=dict(
                depth=depth,
                query='/'.join(self.context.getPhysicalPath())),
            sort_on=sort_on,
            sort_order=sort_order)
        if types:
            query['portal_type'] = types

        return self.context.portal_catalog(query)

    def description(self):
        return self.context.Description()

    def recently_modified(self):
        return self.catalog()[:10]

    def show_search_results(self):
        if 'searchable_text' in self.request:
            searchable_text = self.request.get('searchable_text')
            return bool(len(searchable_text))
        return False

    def generate_date(self, datetimestring, now=None):
        if now is None:
            now = datetime.datetime.now()

        today = now.day
        yesterday = (now - datetime.timedelta(1)).day
        this_month = now.month
        this_year = now.year

        modified = DateTime(datetimestring)
        if modified.month() == this_month and modified.year() == this_year:
            if modified.day() == today:
                return _(u'label_today', default=u'Today, ${time}',
                         mapping={'time': modified.strftime('%H:%M')})

            elif modified.day() == yesterday:
                return _(u'label_yesterday', u'Yesterday, ${time}',
                         mapping={'time': modified.strftime('%H:%M')})
            else:
                return modified.strftime('%d.%m.%Y')
        else:
            return modified.strftime('%d.%m.%Y')

    def translate_readable_date(self, value):
        return helper.translate_state(self.context, value)

    def readable_author(self, item, author):
        return helper.readable_author(item, author)

    def render_sublisting(self):
        return self.context.restrictedTraverse('@@overview_sublisting')()



from DateTime import DateTime
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.workspace.testing import FTW_WORKSPACE_FUNCTIONAL_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from unittest2 import TestCase


class TestFtwMeetingIntegration(TestCase):

    layer = FTW_WORKSPACE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

        self.workspace = create(Builder('workspace').titled('Workspace'))

    def test_meeding_is_addable(self):
        meeting = create(Builder('meeting')
                         .within(self.workspace)
                         .titled('Test Meeting')
                         .having(start_date=DateTime('08/20/2010 08:00'),
                                 end_date=DateTime('08/20/2010 10:00')))

        self.assertEquals([meeting.getId()], self.workspace.objectIds())

    @browsing
    def test_meeting_is_listed_in_on_tab(self, browser):
        meeting = create(Builder('meeting')
                         .within(self.workspace)
                         .titled('Test Meeting')
                         .having(start_date=DateTime('08/20/2010 08:00'),
                                 end_date=DateTime('08/20/2010 10:00')))

        browser.login().visit(self.workspace, view='tabbedview_view-events')

        self.assertEquals(meeting.start().strftime('%d.%m.%Y'),
                          browser.css('table tbody td')[0].text,
                          'Expect start date in first column.')
        self.assertEquals(meeting.Title(),
                          browser.css('table tbody td')[1].text,
                          'Expect title in second column.')



from zope.interface import Interface


class IWorkspace(Interface):
    """A type for collaborative spaces."""


class ITabbedViewFolder(Interface):
    """Tabbed view folder marker interface.
    """


class IWorkspaceDetailsListingProvider(Interface):
    """A adapter providing a listing (LaTeX) for the workspace details PDF.
    """

    def __init__(context, request, layout, view):
        """Adapts context, request, layout and view.
        """

    def get_title():
        """Returns the title of the listing (LaTeX).
        """

    def get_listing():
        """Returns the complete listing (LaTeX).
        """

    def get_sort_key():
        """A integer number which is used for sorting the listings.
        Small numbers are at the top, big numbers at the bottom.
        """


class IWorkspaceLayer(Interface):
    """Request marker interface for ftw.workspace"""


class IWorkspacePreview(Interface):
    """Generic preview adapter"""

    def __init__(context, request):
        """Adapts context and request"""

    def preview():
        """Renders the preview, usually an image"""

    def full_url():
        """URL of the image showing the large scale"""

    def get_scale_properties():
        """Returns the scale defined in configuration registry"""



from ftw.builder import Builder
from ftw.builder import create
from ftw.workspace.interfaces import IWorkspacePreview
from ftw.workspace.testing import FTW_WORKSPACE_INTEGRATION_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from pyquery import PyQuery
from unittest2 import TestCase
from zope.component import queryMultiAdapter


class TestPreview(TestCase):

    layer = FTW_WORKSPACE_INTEGRATION_TESTING

    def setUp(self):
        portal = self.layer['portal']
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)

        self.workspace = create(Builder('workspace'))
        self.tab = self.workspace.restrictedTraverse(
           '@@tabbedview_view-documents')

        portal.portal_types.get(
            'Workspace').allowed_content_types = ('File, Image')

    def test_show_pills_on_document_tab(self):
        create(Builder('image')
            .within(self.workspace)
            .with_dummy_content())

        doc = PyQuery(self.tab())
        self.assertTrue(doc('.ViewChooser'))



from Products.CMFCore.utils import getToolByName
from Products.ZCatalog.Catalog import CatalogError
from ftw.workspace.config import INDEXES, METADATA


def add_indexes(site):
    """Add our indexes to the catalog.

    Doing it here instead of in profiles/default/catalog.xml means we
    do not need to reindex those indexes after every reinstall.
    """

    context = site.getSite()
    catalog = getToolByName(context, 'portal_catalog')
    indexes = catalog.indexes()

    for name, meta_type in INDEXES:
        if name not in indexes:
            catalog.addIndex(name, meta_type)
        if name in METADATA:
            try:
                catalog.manage_addColumn(name)
            except CatalogError:
                pass



from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.pdfgenerator.interfaces import ILaTeXLayout
from ftw.pdfgenerator.view import MakoLaTeXView
from ftw.table import helper
from ftw.workspace import _
from ftw.workspace.interfaces import IWorkspace
from ftw.workspace.interfaces import IWorkspaceDetailsListingProvider
from zope.component import adapts
from zope.component import getAdapters
from zope.i18n import translate
from zope.interface import Interface
from zope.interface import implements
from ftw.workspace.utils import has_ftwfile


class WorkspaceDetailsView(MakoLaTeXView):
    adapts(IWorkspace, Interface, ILaTeXLayout)

    template_directories = ['templates']
    template_name = 'details.tex'

    def get_render_arguments(self):
        self.layout.use_package('pdflscape')
        self.layout.use_package('longtable')
        self.layout.use_package('array,supertabular')

        args = {}
        args.update(self.get_labels())
        args.update(self.get_workspace_metadata())
        args['listings'] = self.get_listings()

        return args

    def get_labels(self):
        labels = {
            'labelWorkspace': _(u'latex_label_workspace', u'Workspace'),
            'labelTitle': _(u'latex_label_title', u'Title'),
            'labelOwner': _(u'latex_label_owner', u'Owner'),
            'labelDescription': _(u'latex_label_description', u'Description'),
            }

        for key, value in labels.items():
            labels[key] = translate(value, context=self.request)

        return labels

    def get_workspace_metadata(self):
        return {
            'workspaceTitle': self.convert(self.context.Title()),
            'workspaceOwner': self.convert(self.get_owner()),
            'workspaceDescription': self.convert(self.context.Description()),
            'workspaceText': self.convert(self.context.getText()),
            }

    def get_owner(self):
        userid = self.context.getOwner(0).getId()

        # Could be None, if there user no longer exists
        if userid is None:
            return ''

        acl_users = getToolByName(self.context, 'acl_users')
        user = acl_users.getUserById(userid)
        return user and user.getProperty('fullname', userid) or userid

    def get_listings(self):
        providers = getAdapters((
                self.context, self.request, self.layout, self),
                                IWorkspaceDetailsListingProvider)
        providers = [provider for name, provider in providers]
        providers.sort(key=lambda p: p.get_sort_key())

        listings = []

        for provider in providers:
            title = provider.get_title()
            latex = provider.get_listing()
            if latex is not None:
                listings.append((title, latex))

        return listings


class FilesListing(object):
    implements(IWorkspaceDetailsListingProvider)
    adapts(IWorkspace, Interface, ILaTeXLayout, Interface)

    template = ViewPageTemplateFile('templates/files_listing.pt')

    def __init__(self, context, request, layout, view):
        self.context = context
        self.request = request
        self.layout = layout
        self.view = view

    def get_sort_key(self):
        return 10

    def get_title(self):
        return translate(_(u'latex_label_files', u'Files'),
                         context=self.request)

    def get_listing(self):
        if len(self._brains()) == 0:
            return None
        else:
            return self.view.convert(self.template())

    def get_items(self):
        """Returns all items to be displayed.
        """
        ftwfile = has_ftwfile(self.context)
        for brain in self._brains():
            yield {'title': brain.Title,
                   'effective': helper.readable_date(
                        brain, getattr(
                        brain, ftwfile and 'documentDate' or 'effective')),
                   'modified': helper.readable_date(
                        brain, getattr(brain, 'modified')),
                   'creator': self.get_creator(brain),
                   }

    def _brains(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        query = {'path': '/'.join(self.context.getPhysicalPath()),
                 'portal_type': ['File'],
                 'sort_on': 'created',
                 'sort_order': 'reverse'}

        return catalog(query)

    def get_creator(self, brain):
        userid = brain.Creator
        acl_users = getToolByName(self.context, 'acl_users')
        user = acl_users.getUserById(userid)
        return user and user.getProperty('fullname', userid) or userid



from ftw.upgrade import UpgradeStep


class InstallNewFeatures(UpgradeStep):
    """
    Enable filtering on overview.
    Truncate long text in recently modified listing.
    Use 'today + time' and 'yesterday + time' in recently modified listing.
    """

    def __call__(self):
        self.setup_install_profile(
            'profile-collective.js.jqsmartTruncation:default')

        self.setup_install_profile(
            'profile-ftw.workspace.upgrades:1700')



from ftw.file.interfaces import IFile
from ftw.workspace.browser.previews.image import ImagePreview
from zope.component import adapts
from zope.interface import Interface


class ImagePreview(ImagePreview):
    """Image preview"""

    adapts(IFile, Interface)



from ftw.lawgiver.tests.base import WorkflowTest
from ftw.workspace.testing import FTW_WORKSPACE_INTEGRATION_TESTING


class TestWorkflowSpecificationWorkspaceContent(WorkflowTest):

    workflow_path = '../profiles/default/workflows/workspace_content_workflow'
    layer = FTW_WORKSPACE_INTEGRATION_TESTING


class TestWorkflowSpecificationWorkspace(WorkflowTest):

    workflow_path = '../profiles/default/workflows/workspace_workflow'
    layer = FTW_WORKSPACE_INTEGRATION_TESTING


class TestWorkflowSpecificationWorkspaces(WorkflowTest):

    workflow_path = '../profiles/default/workflows/workspaces_workflow'
    layer = FTW_WORKSPACE_INTEGRATION_TESTING



from ftw.builder import Builder
from ftw.builder import create
from ftw.workspace.quick_upload import generate_id
from ftw.workspace.testing import FTW_WORKSPACE_INTEGRATION_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from unittest2 import TestCase


class TestQuickUploadIdGenerator(TestCase):

    layer = FTW_WORKSPACE_INTEGRATION_TESTING

    def setUp(self):
        super(TestQuickUploadIdGenerator, self).setUp()

        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

    def test_name_starting_with_underscore(self):
        name = u'_test'
        self.assertEquals(u'test', generate_id(name, self.portal))

    def test_existing_name(self):
        create(Builder('file').titled('test'))
        name = u'test'
        self.assertEquals(u'test-1', generate_id(name, self.portal))

    def test_name_with_umlauts(self):
        create(Builder('file').titled('test'))
        name = u't\xf6st'
        self.assertEquals(u'tost', generate_id(name, self.portal))

    def test_name_with_spaces(self):
        name = u'test test'
        self.assertEquals(u'test-test', generate_id(name, self.portal))



from Acquisition._Acquisition import aq_parent, aq_base
from ftw.workspace import arbeitsraumMessageFactory as _
from Products.CMFCore.utils import getToolByName
import StringIO
import xlsxwriter


ROLES_WHITE_LIST = ['Owner']


def format_date(date):
    if not date:
        return ''
    if date.year() <= 1900:
        return ''
    else:
        return date.strftime('%d.%m.%Y %H:%M')


def fullname(context, uid):
    member = context.portal_membership.getMemberById(uid)
    if member:
        fullname = member.getProperty('fullname', member.getId())
        if fullname:
            return fullname
        return member.getId()
    return uid


def get_participants(context):
    """Get the participants (which are local_roles).
    """
    mtool = getToolByName(context, 'portal_membership')
    users = []

    for userid, roles in get_roles_settings(context):

        member = mtool.getMemberById(userid)

        # skip groups
        if member is not None:
            email = member.getProperty('email', '')
            name = member.getProperty('fullname', '')

            all_roles = roles.keys()
            inherited_roles = [r for r, v in roles.items()
                               if v]

            item = dict(
                userid=userid,
                roles=all_roles,
                inherited_roles=inherited_roles)

            if name:
                item['name'] = name.decode('utf-8')
            else:
                item['name'] = userid.decode('utf-8')

            if email:
                item['email'] = email.decode('utf-8')
            else:
                item['email'] = ''

            if item.get('roles') or item.get('inherited_roles'):
                users.append(item)

    users.sort(key=lambda item: item['name'].lower())
    return users


def get_roles_settings(context):
    portal = getToolByName(context, 'portal_url').getPortalObject()
    result = {}

    def get_roles(context, acquired=False):
        for userid, roles in context.get_local_roles():
            roles = list(roles)
            if 'Owner' in roles and acquired:
                roles.remove('Owner')

            if userid not in result:
                result[userid] = dict((role, acquired) for role in roles)
            else:
                current_roles = result[userid]
                for role in roles:
                    if role not in current_roles:
                        current_roles[role] = acquired

        parent = aq_parent(context)
        if parent != portal and inherited(context):
            get_roles(parent, acquired=True)

    get_roles(context)
    result = list(result.items())
    return result


def inherited(context):
    """Return True if local roles are inherited on the context.
    """
    return not bool(
        getattr(aq_base(context), '__ac_local_roles_block__', None))


def get_header(context):
    return [
        context.translate(
            _(u'export_heading_user_id', default=u'User ID')),
        context.translate(
            _(u'export_heading_full_name', default=u'Full name')),
        context.translate(
            _(u'export_heading_email', default=u'E-Mail')),
        context.translate(
            _(u'export_heading_roles', default=u'Roles')),
    ]


def get_data(context):
    data = []

    for participant in get_participants(context):
        row = [
            participant['userid'].encode('utf-8'),
            participant['name'].encode('utf-8'),
            participant['email'].encode('utf-8'),
            ', '.join(participant['roles']),
        ]
        data.append(row)

    return data


def create_xlsx(header=None, data=None):
    if not header:
        header = []
    if not data:
        data = []

    # Create an in-memory output file for the new workbook.
    output = StringIO.StringIO()

    # Even though the final file will be in memory the module uses temp
    # files during assembly for efficiency. To avoid this on servers that
    # don't allow temp files, for example the Google APP Engine, set the
    # 'in_memory' constructor option to True.
    workbook = xlsxwriter.Workbook(output, {'in_memory': False})
    worksheet = workbook.add_worksheet()

    # Add a bold format for the header.
    bold = workbook.add_format({'bold': True})

    # Write the header.
    for index, item in enumerate(header):
        worksheet.write(0, index, item, bold)

    # Write the data (which is a nested list).
    for row_index, row_item in enumerate(data):
        for column_index, column_item in enumerate(row_item):
            worksheet.write_string(
                row_index+1, column_index, column_item.decode('utf-8'))

    # Set the auto filter only if there is something to filter.
    if header and data:
        worksheet.autofilter(0, 0, len(data)+1, len(data[0])-1)

    workbook.close()

    # Rewind the buffer.
    output.seek(0)

    return output



from ftw.upgrade import UpgradeStep


class InstallPreview(UpgradeStep):

    def __call__(self):
        self.setup_install_profile('profile-ftw.colorbox:default')

        self.setup_install_profile(
            'profile-ftw.workspace.upgrades:1704',
            steps=['propertiestool', 'cssregistry', 'jsregistry'])



from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.workspace.testing import FTW_WORKSPACE_FUNCTIONAL_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from Products.CMFCore.utils import getToolByName
from StringIO import StringIO
from unittest2 import TestCase
from xlrd import open_workbook
from zipfile import ZipFile
import transaction


class TestWorkspaceZipExport(TestCase):

    layer = FTW_WORKSPACE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        self.user = create(Builder('user')
                           .named('may', 'exists')
                           .with_roles('Manager'))

        login(self.portal, self.user.getId())

        self.workspace = create(Builder('workspace')
                                .titled('Workspace')
                                .having(description='Description'))

        create(Builder('user')
               .named('Hugo', 'Boss')
               .with_roles('Editor', on=self.workspace))

    def get_zip_content(self, browser):
        browser.login().visit(self.workspace, view='zip_export')

        self.assertEquals('application/zip', browser.headers['Content-Type'])

        zipfile = ZipFile(StringIO(browser.contents))
        self.assertEquals(['workspace.pdf', 'participants.xlsx'],
                          zipfile.namelist())

        xlsx = zipfile.read('participants.xlsx')

        workbook = open_workbook(file_contents=xlsx)
        sheet = workbook.sheet_by_index(0)

        data = map(sheet.row_values, range(sheet.nrows))
        headers = data.pop(0)
        data = map(
            lambda row: dict(zip(headers, row)),
            data
        )
        return data

    @browsing
    def test_zip_export_action_is_available(self, browser):
        browser.login().visit(self.workspace)
        self.assertEquals(
            'Export as Zip',
            browser.css('#plone-contentmenu-actions-zipexport').first.text,
            'Expect the zip export action on workspace')

    @browsing
    def test_zip_export_integration(self, browser):
        """
        Makes sure the zip file contains a spreadsheet with the correct
        participants.
        """

        data = self.get_zip_content(browser)
        self.maxDiff = None
        self.assertEquals(
            first=[
                {
                    u'E-Mail': u'hugo@boss.com',
                    u'User ID': u'hugo.boss',
                    u'Full name': u'Boss Hugo',
                    u'Roles': u'Editor'
                },
                {
                    u'E-Mail': u'may@exists.com',
                    u'User ID': u'may.exists',
                    u'Full name': u'Exists May',
                    u'Roles': u'Owner, Contributor, Editor, Reader'
                },
            ],
            second=data
        )

    @browsing
    def test_zip_export_works_if_owner_no_longer_exists(self, browser):
        """
        Makes sure the zip file contains a spreadsheet with the correct
        participants.
        """

        # Remove owner of Workspace
        mtool = getToolByName(self.portal, 'portal_membership')
        mtool.deleteMembers((self.user.getId(),))
        transaction.commit()

        data = self.get_zip_content(browser)

        self.maxDiff = None
        self.assertEquals(
            first=[
                {
                    u'E-Mail': u'hugo@boss.com',
                    u'User ID': u'hugo.boss',
                    u'Full name': u'Boss Hugo',
                    u'Roles': u'Editor'
                },
            ],
            second=data
        )



from ftw.builder import Builder
from ftw.builder import create
from ftw.participation.interfaces import IParticipationSupport
from ftw.testbrowser import browsing
from ftw.workspace.testing import FTW_WORKSPACE_FUNCTIONAL_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from unittest2 import TestCase


class TestFtwParticipationIntegration(TestCase):

    layer = FTW_WORKSPACE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

        self.workspace = create(Builder('workspace').titled('Workspace'))

    @browsing
    def test_tab(self, browser):
        browser.login().visit(self.workspace)
        self.assertEquals('participants',
                          browser.css('#tab-participants').first.text,
                          'Expect the participation tab on workspaces.')

    def test_participation_support_on_workspace(self):
        self.assertTrue(IParticipationSupport.providedBy(self.workspace),
                        'No participation support found :-(')



from ftw.dictstorage.interfaces import IDictStorage
from ftw.upgrade import UpgradeStep
from ftw.upgrade.progresslogger import ProgressLogger


PREFIXES = [
    'ftw.tabbedview-Workspace-tabbedview_view-documents-',
    'ftw.tabbedview-TabbedViewFolder-tabbedview_view-documents-',
    'ftw.tabbedview-Folder-tabbedview_view-documents-',
    ]


class ResetExtJSConfig(UpgradeStep):

    def __call__(self):
        workspace = self.get_any_workspace()
        if not workspace:
            return
        view = workspace.unrestrictedTraverse('tabbedview_view-documents')
        storage = IDictStorage(view)
        keys = storage.storage.storage.keys()[:]

        for key in ProgressLogger('Reset documents tab state', keys):
            for PREFIX in PREFIXES:
                if key.startswith(PREFIX):
                    del storage[key]

    def get_any_workspace(self):
        query = {'portal_type': 'Workspace'}
        brains = self.catalog_unrestricted_search(query)
        if not brains:
            return None
        return self.catalog_unrestricted_get_object(brains[0])



from ftw.builder import builder_registry
from ftw.builder.archetypes import ArchetypesBuilder


class WorkspaceBuilder(ArchetypesBuilder):
    portal_type = 'Workspace'


class TabbedViewFolderBuilder(ArchetypesBuilder):
    portal_type = 'TabbedViewFolder'


builder_registry.register('workspace', WorkspaceBuilder)
builder_registry.register('TabbedViewFolder', TabbedViewFolderBuilder)



from ftw.pdfgenerator.interfaces import IBuilder
from ftw.pdfgenerator.layout.customizable import CustomizableLayout
from ftw.workspace import _
from ftw.workspace.interfaces import IWorkspace
from zope.component import adapts
from zope.i18n import translate
from zope.interface import Interface


class WorkspaceLayout(CustomizableLayout):
    adapts(IWorkspace, Interface, IBuilder)

    template_directories = ['templates']
    template_name = 'layout.tex'

    def get_render_arguments(self):
        labels = {
            'pageLabel': _(u'latex_page', default=u'Page'),
            'printedLabel': _(u'latex_printed', default=u'Printed'),
            }

        for key, value in labels.items():
            labels[key] = translate(value, context=self.request)

        return labels

    def before_render_hook(self):
        self.use_babel()
        self.use_package('inputenc', options='utf8', append_options=False)
        self.use_package('fontenc', options='T1', append_options=False)
        self.use_package('ae,aecompl')
        self.use_package(
            'geometry', options='left=35mm,right=20mm,top=20mm,bottom=25mm',
            append_options=False)
        self.use_package(
            'hyperref',
            options='colorlinks=false,breaklinks=true,'
            'linkcolor=black,pdfborder={0 0 0}',
            append_options=False)

        self.use_package('helvet')
        self.use_package('titlesec', 'compact')
        self.use_package('fancyhdr')
        self.use_package('enumitem')
        self.use_package('lastpage')
        self.use_package('scrtime')



from setuptools import setup, find_packages
import os

version = '4.1.0'

tests_require = [
    'egov.contactdirectory',
    'ftw.builder',
    'ftw.pdfgenerator',
    'ftw.testbrowser',
    'ftw.testing',
    'plone.app.testing',
    'pyquery',
    'xlrd',
]

contact_require = [
    'egov.contactdirectory'
]

setup(name='ftw.workspace',
      version=version,
      description='A project folder for plone.',
      long_description=open('README.rst').read() + '\n' + \
          open(os.path.join('docs', 'HISTORY.txt')).read(),

      # Get more strings from
      # http://www.python.org/pypi?%3Aaction=list_classifiers

      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.3',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],

      keywords='ftw workspace project folder plone',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      url='https://github.com/4teamwork/ftw.workspace',
      license='GPL2',

      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw'],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
        'collective.deletepermission',
        'ftw.activity',
        'ftw.calendar',
        'ftw.colorbox',
        'ftw.file',
        'ftw.lawgiver',
        'ftw.meeting [pdf, zipexport, calendar]',
        'ftw.notification.base',
        'ftw.notification.email',
        'ftw.participation',
        'ftw.pdfgenerator',
        'ftw.tabbedview[extjs, quickupload]',
        'ftw.upgrade',
        'ftw.zipexport',
        'plone.formwidget.contenttree',
        'plone.namedfile',
        'plone.principalsource',
        'Products.CMFPlacefulWorkflow',
        'setuptools',
        'XlsxWriter',
        'z3c.relationfield',
        # -*- Extra requirements: -*-
        ],

      tests_require=tests_require,
      extras_require=dict(tests=tests_require,
                          zip_export=[],  # For backwards compatibility
                          contact=contact_require),

      test_suite='ftw.workspace.tests.test_docs.test_suite',
      entry_points='''
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      ''',
      )



from Products.CMFCore.permissions import ManagePortal
from Products.ATContentTypes.content.schemata import finalizeATCTSchema


def finalizeWorkspaceSchema(schema, folderish=False,
                            moveDiscussion=True, portletLinkTitle=False):
    """Finalizes an ATCT type schema to alter some fields
    """

    finalizeATCTSchema(schema, folderish, moveDiscussion)

    schema.moveField('relatedItems', pos='bottom')
    if folderish:
        schema['relatedItems'].widget.visible['edit'] = 'invisible'
    schema.moveField('excludeFromNav', after='allowDiscussion')
    if moveDiscussion:
        schema.moveField('allowDiscussion', after='relatedItems')

    # Categorization
    if 'subject' in schema:
        schema.changeSchemataForField('subject', 'settings')
    if 'relatedItems' in schema:
        schema.changeSchemataForField('relatedItems', 'settings')
    if 'location' in schema:
        schema.changeSchemataForField('location', 'settings')
        schema['location'].widget.visible = -1
    if 'language' in schema:
        schema.changeSchemataForField('language', 'settings')
        schema['language'].widget.visible = -1

    # Dates
    if 'effectiveDate' in schema:
        schema.changeSchemataForField('effectiveDate', 'default')
        schema['effectiveDate'].widget.visible = -1
    if 'expirationDate' in schema:
        schema.changeSchemataForField('expirationDate', 'default')
        schema['expirationDate'].write_permission = ManagePortal
    if 'creation_date' in schema:
        schema.changeSchemataForField('creation_date', 'settings')
    if 'modification_date' in schema:
        schema.changeSchemataForField('modification_date', 'settings')

    # Ownership
    if 'creators' in schema:
        schema.changeSchemataForField('creators', 'settings')
        schema['creators'].write_permission = ManagePortal
    if 'contributors' in schema:
        schema.changeSchemataForField('contributors', 'settings')
        schema['contributors'].widget.visible = -1
    if 'rights' in schema:
        schema.changeSchemataForField('rights', 'settings')
        schema['rights'].widget.visible = -1

    # Settings
    if 'allowDiscussion' in schema:
        schema.changeSchemataForField('allowDiscussion', 'settings')
    if 'excludeFromNav' in schema:
        schema.changeSchemataForField('excludeFromNav', 'settings')
    if 'nextPreviousEnabled' in schema:
        schema.changeSchemataForField('nextPreviousEnabled', 'settings')
        schema['nextPreviousEnabled'].widget.visible = -1

    settings_fields = [schema[key] for key in schema.keys()
                            if schema[key].schemata == 'settings']
    for field in settings_fields:
        field.write_permission = ManagePortal

    return schema



from ftw.builder import Builder
from ftw.builder import create
from ftw.workspace.testing import FTW_WORKSPACE_INTEGRATION_TESTING
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.registry.interfaces import IRegistry
from unittest2 import TestCase
from zope.component import getUtility


class TestAutoRoles(TestCase):

    layer = FTW_WORKSPACE_INTEGRATION_TESTING

    def setUp(self):
        portal = self.layer['portal']
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)

    def test_roles_are_automaticall_assigned_on_workspace_creation(self):
        workspace = create(Builder('workspace'))
        roles = set(dict(workspace.get_local_roles()).get(TEST_USER_ID))

        self.assertEquals(
            set(['Owner', 'Contributor', 'Editor', 'Reader']),
            roles,
            'The configured roles should be automatically granted to the owner.')

    def test_auto_roles_are_configurable_with_registry_entry(self):
        registry = getUtility(IRegistry)
        roles = registry['ftw.workspace.auto_roles'] = [u'Editor']

        workspace = create(Builder('workspace'))
        roles = set(dict(workspace.get_local_roles()).get(TEST_USER_ID))

        self.assertEquals(
            set(['Owner', 'Editor']),
            roles,
            'Only the configured auto_roles should be granted to the owner.')
